﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Core;
using HeuristicLab.Problems.BinPacking.Dimensions;
using HeuristicLab.Problems.BinPacking.PackingBin;
using HeuristicLab.Problems.BinPacking.PackingItem;
using HeuristicLab.Encodings.PackingEncoding.PackingPlan;
using HeuristicLab.Encodings.PermutationEncoding;
using HeuristicLab.Common;
using HeuristicLab.Data;
using HeuristicLab.Collections;
using HeuristicLab.Problems.BinPacking.Interfaces;
using HeuristicLab.Encodings.PackingEncoding.GroupingVector;
using HeuristicLab.Encodings.IntegerVectorEncoding;

namespace HeuristicLab.Problems.BinPacking.Decoders {
  [Item("Lowest Gap Fill Grouping-Vector Decoder", "<Description missing...>")]
  [StorableClass]
  public class IB2DLowestGapFillGroupingVectorDecoder : IdenticalBinPackingSolutionDecoder<
      TwoDimensionalPacking,
      RectangularPackingBin,
      RectangularPackingItem,
      PackingPlan<TwoDimensionalPacking, RectangularPackingBin, RectangularPackingItem>> {

    public IB2DLowestGapFillGroupingVectorDecoder()
      : base() {
        //EncodedSolutionParameter.ActualName = "EncodedSolution";
    }
    [StorableConstructor]
    protected IB2DLowestGapFillGroupingVectorDecoder(bool deserializing) : base(deserializing) { }
    protected IB2DLowestGapFillGroupingVectorDecoder(IB2DLowestGapFillGroupingVectorDecoder original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new IB2DLowestGapFillGroupingVectorDecoder(this, cloner);
    }


    protected override PackingPlan<TwoDimensionalPacking, RectangularPackingBin, RectangularPackingItem> CreatePackingPlanFromEncoding(IPackingSolutionEncoding encodedSolution) {

      var solution = encodedSolution as IntegerVector;
      if (solution == null) throw new InvalidOperationException("Encoding is not of type PackingSequence");

      var binMeasures = PackingBinMeasuresParameter.ActualValue;
      var itemMeasures = PackingItemMeasuresParameter.ActualValue;

      ObservableDictionary<int, TwoDimensionalPacking> packingPositions = new ObservableDictionary<int, TwoDimensionalPacking>();

      int nrOfBins = solution.Max();

      //Get all indexes of items for every bin according to grouping-vector
      //It is assumed, that at this point the item-indexes are sorted according to their item-size!
      Dictionary<int, List<int>> unpackedItemIndexesPerBin              = new Dictionary<int, List<int>>();

      for (int i = 0; i < nrOfBins; i++) {
        unpackedItemIndexesPerBin[i] = solution
          .Select((Value, Index) => new { Value, Index })
          .Where (temp => temp.Value == i)
          .Select (temp => temp.Index).ToList();  
      }



      //Steps 1-6
      //2
      int currentBin = 0;    
      
      //3
      //Take the first position from the first bin
      TwoDimensionalPacking currentPoint = new TwoDimensionalPacking(currentBin, 0, 0);
   
      //4
      //Assuming that Height is always the shorter edge due to normalizing..
      int currentGap = binMeasures.Height;

      //5
      //Take the first (and largest) item for the first bin
      int currentItemIndex = unpackedItemIndexesPerBin[currentBin][0];
      RectangularPackingItem currentItem = itemMeasures[currentItemIndex];
      packingPositions[currentItemIndex] = currentPoint;
      //6
      unpackedItemIndexesPerBin[currentBin].RemoveAt(0);


      //Steps 7-13          

      //TODO: the first parameter (bin-list) is not finished yet.. it should resemble the used bins in the packing plan..
      PackingPlan<TwoDimensionalPacking, RectangularPackingBin, RectangularPackingItem> result =
        new PackingPlan<TwoDimensionalPacking, RectangularPackingBin, RectangularPackingItem>(
          binMeasures, 
          itemMeasures);

      result.PackingItemPositions = packingPositions;
      return result;
    }

    //protected override TwoDimensionalPacking FindFeasiblePositionForItem(RectangularPackingBin binMeasures, int binNr, RectangularPackingItem currentItem, ObservableDictionary<int, TwoDimensionalPacking> itemPositions, ItemList<RectangularPackingItem> itemMeasures) {
    //  //Starting-position at upper right corner (=left bottom point of item-rectangle is at position item.width,item.height)
    //  TwoDimensionalPacking currentPosition = new TwoDimensionalPacking(binNr, binMeasures.Width - currentItem.Width, binMeasures.Height - currentItem.Height);           
    //  //Slide the item as far as possible to the bottom
    //  while (IsPositionFeasible(binMeasures, binNr, currentItem, MoveDown(currentPosition), itemPositions, itemMeasures)) {
    //    currentPosition = MoveDown(currentPosition);
    //  }
    //  //Slide the item as far as possible to the left
    //  while (IsPositionFeasible(binMeasures, binNr, currentItem, MoveLeft(currentPosition), itemPositions, itemMeasures)) {
    //    currentPosition = MoveLeft(currentPosition);
    //  }

    //  return IsPositionFeasible(binMeasures, binNr, currentItem, currentPosition, itemPositions, itemMeasures) ? currentPosition : null;
    //}

    //private bool IsPositionFeasible(RectangularPackingBin binMeasures, int binNr, RectangularPackingItem currentItem,
    //  TwoDimensionalPacking currentPosition, ObservableDictionary<int, TwoDimensionalPacking> itemPositions, ItemList<RectangularPackingItem> itemMeasures) {
    //  //In this case feasability is defined as following: 1. the item fits into the bin-borders; 2. the item does not collide with another already packed item
    //  if (!binMeasures.Encloses(currentPosition, currentItem))
    //    return false;
    //  var itemsWithPositions = itemMeasures.
    //    Select((Value, Index) => new { Value, Index }).
    //    Where(s => itemPositions.ContainsKey(s.Index)).
    //    Select((s) => new { Measures = itemMeasures[s.Index], Position = itemPositions[s.Index]});

    //  foreach (var itemWithPosition in itemsWithPositions) {
    //    if (itemWithPosition.Position.AssignedBin == binNr && itemWithPosition.Measures.Overlaps(itemWithPosition.Position, currentPosition, currentItem))
    //      return false;
    //  }
    //  return true;
    //}

    //private TwoDimensionalPacking MoveLeft(TwoDimensionalPacking original) {
    //  return new TwoDimensionalPacking(original.AssignedBin, original.X - 1, original.Y);
    //}
    //private TwoDimensionalPacking MoveDown(TwoDimensionalPacking original) {
    //  return new TwoDimensionalPacking(original.AssignedBin, original.X, original.Y - 1);
    //}
  }
}
