﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Optimization;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.PackingEncoding.GroupingVector {
  [Item("Exhaustive Target Bin Single Grouping Move Generator", "Generates all possible single grouping moves for a specified target bin from a given groupingVector.")]
  [StorableClass]
  public class ExhaustiveTargetBinSingleGroupingMoveGenerator : SingleGroupingMoveGenerator, IExhaustiveMoveGenerator {
    [StorableConstructor]
    protected ExhaustiveTargetBinSingleGroupingMoveGenerator(bool deserializing) : base(deserializing) { }
    protected ExhaustiveTargetBinSingleGroupingMoveGenerator(ExhaustiveTargetBinSingleGroupingMoveGenerator original, Cloner cloner) : base(original, cloner) { }
    public ExhaustiveTargetBinSingleGroupingMoveGenerator() : base() { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new ExhaustiveTargetBinSingleGroupingMoveGenerator(this, cloner);
    }

    private static IEnumerable<SingleGroupingMove> Generate(GroupingVectorEncoding groupingVector, int lastBinNr, List<int> indexesOfItemsInLastBin) {
      for (int i = 0; i < indexesOfItemsInLastBin.Count; i++)
        for (int j = 0; j < lastBinNr; j++) {
          yield return new SingleGroupingMove(i, j, groupingVector);
        }
    }

    public static SingleGroupingMove[] Apply(GroupingVectorEncoding groupingVector) {
      int lastBinNr = 0;
      foreach (int binNr in groupingVector.GroupingVector) {
        if (binNr > lastBinNr)
          lastBinNr = binNr;
      }
      int nrOfBins = lastBinNr + 1;
      List<int> indexesOfItemsInLastBin = new List<int>();
      for (int i = 0; i < groupingVector.GroupingVector.Length; i++ ) {
        if (groupingVector.GroupingVector[i] == lastBinNr)
          indexesOfItemsInLastBin.Add(i);
      }

      int totalMoves = indexesOfItemsInLastBin.Count * (nrOfBins - 1);
      SingleGroupingMove[] moves = new SingleGroupingMove[totalMoves];
      int count = 0;
      foreach (SingleGroupingMove move in Generate(groupingVector, lastBinNr, indexesOfItemsInLastBin)) {
        moves[count++] = move;
      }
      return moves;
    }

    protected override SingleGroupingMove[] GenerateMoves(GroupingVectorEncoding groupingVector) {
      return Apply(groupingVector);
    }
  }
}
