﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.IntegerVectorEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.BinPacking.Interfaces;

namespace HeuristicLab.Encodings.PackingEncoding.GroupingVector {
  [Item("Single Grouping Move", "A move on a grouping vector that is specified by a single group-assignment-index.")]
  [StorableClass]
  public class SingleGroupingMove : Item, IPackingMove{
    [Storable]
    public int Index { get; protected set; }
    [Storable]
    public int NewGroup { get; protected set; }
    [Storable]
    public GroupingVectorEncoding GroupingVector { get; protected set; }

    [StorableConstructor]
    protected SingleGroupingMove(bool deserializing) : base(deserializing) { }
    protected SingleGroupingMove(SingleGroupingMove original, Cloner cloner)
      : base(original, cloner) {
      this.Index = original.Index;
      this.NewGroup = original.NewGroup;
      if (original.GroupingVector != null)
        this.GroupingVector = cloner.Clone(original.GroupingVector);
    }
    public SingleGroupingMove(int index, int newGroup, GroupingVectorEncoding groupingVector)
      : base() {
      Index = index;
      GroupingVector = groupingVector;
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new SingleGroupingMove(this, cloner);
    }

    public IPackingSolutionEncoding GetSolutionAfterMove() {
      GroupingVectorEncoding newSolution = new GroupingVectorEncoding();
      newSolution.GroupingVector = new IntegerVector(GroupingVector.GroupingVector);
      newSolution.GroupingVector[Index] = NewGroup;
      return newSolution;
    }
  }
}